@@ -1,12 +1,12 @@
 ;;; wisp-mode.el --- Tools for wisp: the Whitespace-to-Lisp preprocessor
 
-;; Copyright (C) 2013--2016  Arne Babenhauserheide <arne_bab@web.de>
+;; Copyright (C) 2013--2025  Arne Babenhauserheide <arne_bab@web.de>
 ;; Copyright (C) 2015--2016  Kevin W. van Rooijen — indentation and tools
 ;;               from https://github.com/kwrooijen/indy/blob/master/indy.el
 
 ;; Author: Arne Babenhauserheide <arne_bab@web.de>
-;; Package-Version: 1.0.12
-;; Package-Revision: v1.0.12-0-m7c6239a77965
+;; Package-Version: 20251108.2318
+;; Package-Revision: cfebcd5f097f
 ;; Keywords: languages, lisp, scheme
 ;; Homepage: http://www.draketo.de/english/wisp
 ;; Package-Requires: ((emacs "24.4"))
@@ -43,6 +43,12 @@
 ;; 
 ;; ChangeLog:
 ;;
+;;  - 0.4.1  fix linting errors: custom, deprecations, and docstrings.
+;;  - 0.4.0: provide wisp--eval-block (C-M-x)
+;;           to send the current block to a buffer nammed *shell*.
+;;           wisp--find-begin-and-end-of-block-around-region can cope with empty lines.
+;;           wisp--eval-with-geiser converts wisp to regular scheme (with wisp2lisp) and pushes it to geiser.
+;;           thanks to cwebber!
 ;;  - 0.3.0: provide wisp-color-indentation-minor-mode
 ;;           that highlights the indentation levels, following wisp-semantics (period and colon)
 ;;  - 0.2.9: enabled imenu - thanks to Greg Reagle!
@@ -62,6 +68,7 @@
 ;;; Code:
 
 (require 'scheme)
+(require 'comint)
 
 ; see http://www.emacswiki.org/emacs/DerivedMode
 
@@ -142,7 +149,7 @@
 
 (defun wisp--get-current-line ()
   "Get the current line as a string."
-  (buffer-substring-no-properties (point-at-bol) (point-at-eol)))
+  (buffer-substring-no-properties (pos-bol) (pos-eol)))
 
 (defun wisp--current-indent ()
   "Get the amount of indentation spaces if the current line."
@@ -190,7 +197,7 @@ if the current indentation is less than the previous line,
 (defun wisp--backtab ()
   "Cycle through indentations depending on the previous line.
 
-This is the inverse of 'wisp--tab', except that it jums from 0 to
+This is the inverse of `wisp--tab', except that it jums from 0 to
 prev, not to prev+tab."
   (interactive)
   (let* ((curr (wisp--current-indent))
@@ -213,6 +220,32 @@ prev, not to prev+tab."
     (wisp--indent curr)))
 
 
+(defvar wisp--eval-process-target nil)
+(defun wisp--eval-block (arg)
+  "Send the current block to a target buffer (by default *shell*).
+
+Set ARG \\<mapvar> & \\[command] to select the target buffer.
+
+Similar to `eval-defun'."
+  (interactive "P")
+  (if (or arg
+          (not wisp--eval-process-target) ;; unset
+          (not (process-live-p (get-buffer-process
+                                wisp--eval-process-target))))
+      (setq wisp--eval-process-target
+            (completing-read
+             "Shell: "
+             (seq-map (lambda (el) (buffer-name (process-buffer el)))
+                      (process-list)))))
+  (if (not (member wisp--eval-process-target (mapcar #'buffer-name (buffer-list))))
+      (error (concat "There is no buffer named \"" wisp--eval-process-target "\": cannot send the command.
+To eval the current block, please use M-x shell and open a REPL there"))
+    (save-mark-and-excursion
+      
+      (let* ((begin-and-end (wisp--find-begin-and-end-of-block-around-region (point) (point)))
+             (block (string-trim (buffer-substring-no-properties (car begin-and-end) (cdr begin-and-end)))))
+        (process-send-string wisp--eval-process-target block)
+        (process-send-string wisp--eval-process-target "\n\n")))))
 
 ; use this mode automatically
 ;;;###autoload
@@ -237,7 +270,8 @@ prev, not to prev+tab."
   (define-key wisp-mode-map (kbd "C-c i") '("imenu" . imenu))
   (define-key wisp-mode-map (kbd "<tab>") '("indent line" . wisp--tab))
   (define-key wisp-mode-map (kbd "<backtab>") '("unindent line" . wisp--backtab))
-  (define-key wisp-mode-map "\r" '("wisp newline" . wisp--return)))
+  (define-key wisp-mode-map "\r" '("wisp newline" . wisp--return))
+  (define-key wisp-mode-map (kbd "C-M-x") '("eval block in *shell* buffer" . wisp--eval-block)))
 
 ; use this mode automatically
 ;;;###autoload
@@ -277,9 +311,11 @@ prev, not to prev+tab."
     "#EFEFBB" "#ffffd0"
     "#FFCECD" "#ffdede"
     )
-  "Highlight-colors for the current level: First the matching color then the replacement."
+  "Highlight-color change for the current level.
+
+The first is the matching color the second the replacement."
   :group 'wisp
-  :type 'plist)
+  :type '(plist :key-type string :value-type string))
 
 (defcustom wisp--bg-colors
   '( ;; paul tol's pale scheme, the cycled ones become slightly
@@ -314,7 +350,7 @@ prev, not to prev+tab."
     )
   "Background-colors to show the indentation."
   :group 'wisp
-  :type 'list)
+  :type '(list string))
 
 (defun wisp--add-indentation-levels-before (indent levels)
   "Add the indentation level with INDENT or less to the LEVELS."
@@ -323,21 +359,24 @@ prev, not to prev+tab."
     (wisp--add-indentation-levels-before (wisp-prev-indent-lower-than indent) (+ levels 1))))
 
 (defun wisp--current-indentation-level (indent)
-  "Get the indentation level at the INDENT — the number of indentation levels defined before it."
+  "Get the indentation level at the INDENT.
+
+The level is number of indentation levels defined (used) before it."
   (wisp--add-indentation-levels-before indent 1))
 
 (defvar-local wisp--highlight-indentation-overlays '()
-  "Overlays set by wisp indentation highlighting in the current
-  buffer.")
+  "Overlays set by wisp indentation highlighting in the current buffer.")
 
 (defvar-local wisp--current-wisp-highlight-overlays-at-point '()
-  "The overlay the point was in in the last time the
-  wisp--highlight-current-indentation-level was called.")
+  "The overlay the point was in at previous highlighting.
+
+Records the last time the wisp--highlight-current-indentation-level was called.")
 
 (defvar-local wisp--original-colors-dynamic '()
-  "The inverse of wisp--brighter-colors: the original-colors
-  mapped too the brighter ones, filled when colors are
-  replaced.")
+  "The inverse of wisp--brighter-colors.
+
+The original-colors are mapped too the brighter ones, filled when colors are
+replaced.")
 
 (defvar-local wisp--current-highlight-brighter-color nil
   "The color of currently highlighted overlays.")
@@ -349,7 +388,7 @@ prev, not to prev+tab."
   (wisp--highlight-indentation-region (point-min) (point-max)))
 
 (defun wisp--find-begin-and-end-of-block-around-region (begin end)
-  "Search around the current region and return the wisp-block around it."
+  "Return the wisp-block around the current region (BEGIN to END)."
   (let ((begin (if (not begin)
                    (point-min)
                  begin))
@@ -359,15 +398,26 @@ prev, not to prev+tab."
     (save-mark-and-excursion
       (goto-char begin)
       (backward-paragraph)
+      (forward-line 1)
+      ;; move backwards until the first line has indentation zero
+      (while (not (= 0 (wisp--current-indent)))
+        (forward-line -1)
+        (backward-paragraph)
+        (forward-line 1))
       (setq begin (point))
       (goto-char end)
       (forward-paragraph)
+      ;; move forwards until the first line has indentation zero
+      (forward-line 1)
+      (while (not (= 0 (wisp--current-indent)))
+        (forward-paragraph)
+        (forward-line 1))
       (setq end (point))
       (goto-char begin))
     (cons begin end)))
 
 (defun wisp--find-begin-and-end-of-lines-with-same-indentation (position)
-  "Search around the current region and return the wisp-block around it."
+  "Return the wisp-block around POSITION."
   (save-mark-and-excursion
     (let* ((begin (if (not position)
                       (point)
@@ -376,11 +426,11 @@ prev, not to prev+tab."
       (goto-char position)
       (let ((indentation (wisp--current-indent)))
         (while (and (> (point) (point-min)) (equal indentation (wisp--current-indent)))
-          (setq begin (point-at-bol))
+          (setq begin (pos-bol))
           (forward-line -1))
         (forward-line 1)
         (while (and (< (point) (point-max)) (equal indentation (wisp--current-indent)))
-          (setq end (point-at-eol))
+          (setq end (pos-eol))
           (forward-line 1))
         (cons begin end)))))
 
@@ -400,13 +450,13 @@ prev, not to prev+tab."
             (allow-same-indent (looking-at ". ")))
         (forward-line -1)
         (while (and (> (point) (point-min)) (if allow-same-indent (>= indentation (wisp--current-indent)) (> indentation (wisp--current-indent))))
-          (setq begin (point-at-bol))
+          (setq begin (pos-bol))
           (setq indentation (wisp--current-indent))
           (forward-line -1))
         (goto-char end)
         (forward-line 1)
         (while (and (< (point) (point-max)) (< indentation (wisp--current-indent)))
-          (setq end (point-at-eol))
+          (setq end (pos-eol))
           (setq indentation (wisp--current-indent))
           (forward-line 1))
         (cons begin end)))))
@@ -480,7 +530,6 @@ prev, not to prev+tab."
 ;;;###autoload
 (define-minor-mode wisp-color-indentation-minor-mode
   "Mode to colorize the indentation level according to wisp-semanttics."
-  nil nil nil
   :group 'wisp
   :after-hook (if wisp-color-indentation-minor-mode
                   (progn
@@ -494,8 +543,7 @@ prev, not to prev+tab."
 
 ;;;###autoload
 (define-minor-mode wisp-color-highlight-current-indentation-minor-mode
-  "Mode to colorize the indentation level according to wisp-semanttics. THIS IS A WORK IN PROGRESS."
-  nil nil nil
+  "Mode to colorize the indentation level according to wisp-semanttics."
   :group 'wisp
   :after-hook (if wisp-color-highlight-current-indentation-minor-mode
                   (progn
@@ -507,8 +555,7 @@ prev, not to prev+tab."
 
 ;;;###autoload
 (define-minor-mode wisp-color-highlight-current-subtree-minor-mode
-  "Mode to colorize the indentation level according to wisp-semanttics. THIS IS A WORK IN PROGRESS."
-  nil nil nil
+  "Mode to colorize the indentation level according to wisp-semanttics."
   :group 'wisp
   :after-hook (if wisp-color-highlight-current-subtree-minor-mode
                   (progn
@@ -519,23 +566,24 @@ prev, not to prev+tab."
 
 
 (defun wisp--highlight-overlay-color (overlay)
-  "Replace the background color of the OVERLAY with a lighter color from wisp--brighter-colors."
+  "Replace the background color of the OVERLAY from wisp--brighter-colors."
   (let ((color (plist-get (plist-get (overlay-properties overlay) 'face) :background)))
-    (let ((new-color (lax-plist-get wisp--brighter-colors color)))
+    (let ((new-color (plist-get wisp--brighter-colors color)))
       (when new-color
-        (unless (lax-plist-get wisp--original-colors-dynamic new-color)
+        (unless (plist-get wisp--original-colors-dynamic new-color)
           (setq wisp--original-colors-dynamic
-                (lax-plist-put wisp--original-colors-dynamic new-color color)))
+                (plist-put wisp--original-colors-dynamic new-color color)))
         (setq wisp--current-highlight-brighter-color new-color)
         (overlay-put overlay 'face `(:background ,new-color))))))
 
 (defun wisp--overlay-background-color (overlay)
+  "Get the background color from the OVERLAY properties."
   (plist-get (plist-get (overlay-properties overlay) 'face) :background))
 
 (defun wisp--restore-overlay-color (overlay)
-  "Replace the background color of the OVERLAY with its original color from wisp--brighter-colors."
+  "Replace the background color of the OVERLAY with its original color."
   (let ((color (wisp--overlay-background-color overlay)))
-    (let ((original-color (lax-plist-get wisp--original-colors-dynamic color)))
+    (let ((original-color (plist-get wisp--original-colors-dynamic color)))
       (when original-color
         (overlay-put overlay 'face `(:background ,original-color))))))
 
@@ -547,11 +595,11 @@ color of the overlay, the mapped color is set instead."
   (interactive)
   (with-silent-modifications
     (let* ((overlays-with-bg-at-point
-            (remove-if-not
+            (cl-remove-if-not
              (lambda (overlay)
                (let ((color (wisp--overlay-background-color overlay)))
-                 (or (lax-plist-get wisp--brighter-colors color)
-                     (lax-plist-get wisp--original-colors-dynamic color))))
+                 (or (plist-get wisp--brighter-colors color)
+                     (plist-get wisp--original-colors-dynamic color))))
              (overlays-at (point) t)))
            (wisp-overlay-at-point
             (if overlays-with-bg-at-point
@@ -563,18 +611,18 @@ color of the overlay, the mapped color is set instead."
            (end (cdr region))
            (overlays-in-region (overlays-in begin end))
            (current-highlighting-color
-            (lax-plist-get wisp--original-colors-dynamic
+            (plist-get wisp--original-colors-dynamic
                            wisp--current-highlight-brighter-color))
            (overlays-with-same-color
-            (remove-if-not
+            (cl-remove-if-not
              (lambda (overlay)
-               (equalp current-highlighting-color
+               (cl-equalp current-highlighting-color
                        (wisp--overlay-background-color overlay)))
              overlays-in-region))
            (removed-overlays
-            (remove-if
+            (cl-remove-if
              (lambda (overlay)
-               (equalp wisp--current-highlight-brighter-color
+               (cl-equalp wisp--current-highlight-brighter-color
                        (wisp--overlay-background-color overlay)))
              wisp--current-wisp-highlight-overlays-at-point)))
       ;; restore all no longer highlighted overlays
@@ -600,11 +648,11 @@ color of the overlay, the mapped color is set instead."
   (interactive)
   (with-silent-modifications
     (let* ((overlays-with-bg-at-point
-            (remove-if-not
+            (cl-remove-if-not
              (lambda (overlay)
                (let ((color (wisp--overlay-background-color overlay)))
-                 (or (lax-plist-get wisp--brighter-colors color)
-                     (lax-plist-get wisp--original-colors-dynamic color))))
+                 (or (plist-get wisp--brighter-colors color)
+                     (plist-get wisp--original-colors-dynamic color))))
              (overlays-at (point) t)))
            (wisp-overlay-at-point
             (if overlays-with-bg-at-point
@@ -616,18 +664,18 @@ color of the overlay, the mapped color is set instead."
            (end (cdr region))
            (overlays-in-region (overlays-in begin end))
            (current-highlighting-color
-            (lax-plist-get wisp--original-colors-dynamic
+            (plist-get wisp--original-colors-dynamic
                            wisp--current-highlight-brighter-color))
            (overlays-with-same-color
-            (remove-if-not
+            (cl-remove-if-not
              (lambda (overlay)
-               (equalp current-highlighting-color
+               (cl-equalp current-highlighting-color
                        (wisp--overlay-background-color overlay)))
              overlays-in-region))
            (removed-overlays
-            (remove-if
+            (cl-remove-if
              (lambda (overlay)
-               (equalp wisp--current-highlight-brighter-color
+               (cl-equalp wisp--current-highlight-brighter-color
                        (wisp--overlay-background-color overlay)))
              wisp--current-wisp-highlight-overlays-at-point)))
       ;; restore all no longer highlighted overlays
@@ -649,6 +697,7 @@ color of the overlay, the mapped color is set instead."
 
 
 (defun wisp--wisp2lisp ()
+  "Transform the current buffer to Scheme in the buffer *wisp2lisp*."
   (interactive)
   (let ((current-line (line-number-at-pos)))
     (save-excursion
@@ -657,19 +706,19 @@ color of the overlay, the mapped color is set instead."
       (erase-buffer)
       (scheme-mode))
     (call-process "wisp2lisp" nil "*wisp2lisp*" nil (buffer-file-name))
-    (when (called-interactively-p)
+    (when (called-interactively-p 'any)
       (switch-to-buffer-other-window "*wisp2lisp*")
-      (beginning-of-buffer)
+      (goto-char (point-min))
       (forward-line (- current-line 1)))))
 
 (define-key wisp-mode-map (kbd "C-c C-w") 'wisp--wisp2lisp)
 
 (defun wisp--eval-with-geiser ()
+  "Evaluate the current wisp-buffer with Geiser."
   (interactive)
   (require 'geiser)
   (wisp--wisp2lisp)
-  (save-excursion
-    (set-buffer "*wisp2lisp*")
+  (with-current-buffer "*wisp2lisp*"
     (run-geiser 'guile)
     (geiser-eval-buffer)))
 
